//*******************************************************************************
// MODUL OBSLUGI PORTU TWI DLA MIKROKONTROLERA AT89C51RD2
//*******************************************************************************

#include <c51rd2.h>
#include <intrins.h>
#include <TWI.h>

char data TWIbufor[8];      //bufor interfejsu TWI
sbit SDA = P2^6;
sbit SCL = P2^7;


void czekaj(unsigned int t)
{
    unsigned int t1;
 
    for(;t!=0;t--)
        for(t1 = 0x072; t1 != 0; t1--);
}

void TWI_ini(void)          //inicjalizacja interfejsu TWI
{                           //TWI w trybie Master mode
    SDA = 1;                //SDATA=HIGH
    SCL = 1;                //SCLOCK=HIGH
    _nop_();
}

void TWI_start (void)
{                           //Sekwencja startu TWI
                            //SDATA  --\___
                            //SCLOCK -----\__

    SDA=1;
    _nop_();
    SCL=1;
    _nop_();
    SDA=0;                  //Master Mode Data Output: LOW
    _nop_();
    SCL=0;                  //Master Mode Clock Output: LOW
    _nop_();
}

void TWI_stop (void)
{                           //Sekwencja stopu TWI
                            //SDATA  ___/---
                            //SCLOCK _/----

    SDA=0;                  //Master Mode Data Output: LOW
    _nop_();
    SCL=1;                  //Master Mode Clock Output: HIGH
    _nop_();
    SDA=1;                  //Master Mode Data Output: HIGH
}

bit TWIw(unsigned char zn)  //zapis bajtu do TWI
{
    char i;
    bit ack;                //tu bedzie zapamietany stan ACK lub NACK

    for(i=8;i!=0;i--)
    {
        EA = 0;
        zn <<= 1;           //przesun wysylany bit do <CY>
        SDA = CY;           //wyslij bit do pamieci
        EA = 1;
        _nop_();
        SCL = 1;
        _nop_();
        SCL = 0;
        _nop_();
    }
    SCL=1;                  //impuls zegarowy dla bitu potwierdzenia
                            //czytaj potwierdzenie przyjecia danej
    _nop_();                //opoznienie
    if(SDA)
    {
        ack = 1;            //zwrocenie NACK
    }
        else
    {
        ack = 0;            //zwroc ACK
    }
    SCL=0;
    _nop_();
    return (ack);
}

unsigned char TWIr(bit btyp)    //odczyt bajtu z TWI
{                           //btyp=1 => ostatni bajt odczytywany
    char dana, i;

    dana = 0;
    SDA = 1;                //wejscie

    for(i = 8; i != 0; i--)
    {
        dana <<= 1;         //kompletuj dana
        SCL = 1;            //SCLOCK=LOW
        _nop_();
        dana |= SDA;
        SCL = 0;
        _nop_();
    }
    if (btyp)
    {
        SDA=0;              //wystaw potwierdzenie odbioru jesli nie ostatni bajt transmisji
    }
        else
    {
        SDA=1;
    }
    SCL = 1;                //jesli ostatni bajt transmisji to brak potwierdzenia
    _nop_();
    SCL = 0;
    _nop_();
    return (dana);
}

void TWI_zapis(unsigned char adrTWI, unsigned char lz, unsigned char adres)
{                           //zapis lz bajtow do TWI z bufora
    char i;

    TWI_start();
    if(TWIw(adrTWI))        //wywolaj TWI do zapisu (nie zaimplementowano obslugi ew. bladu)
    {
        _nop_();            //ew. obsluga bledu
    }
    if(TWIw(adres))         //wprowadz adres zapisu
    {
        _nop_();            //ew. obsluga bledu
    }
    i = 0;
    for(; lz != 0; lz--)
    {
        if(TWIw(TWIbufor[i++]))
        {
            _nop_();        //ew. obsluga bledu
        }
    }
    TWI_stop();
}

void TWI_odczyt(char adrTWI, char lz, char adres)
{                           //odczyt lz bajtow z TWI do bufora
    char i;

    TWI_start();
    TWIw(adrTWI);           //wywolaj TWI do zapisu (adresu odczytu)
    TWIw(adres);            //wprowadz adres odczytu
    TWI_start();            //powtorzony bit startu
    TWIw(adrTWI | 0x01);    //wywolaj TWI do odczytu (danych)
    i=0;
    for(; lz > 1; lz--)
    {
        TWIbufor[i++] = TWIr(ACK);
    }
    TWIbufor[i++] = TWIr(NACK);
    TWI_stop();
}
